@@ -1,3 +1,5 @@
+# -*- coding: utf-8 -*-
+
 # Licensed under the MIT license
 # http://opensource.org/licenses/mit-license.php
 
@@ -9,6 +11,7 @@ import shutil
 import time
 import re
 from datetime import datetime
+import urllib2
 
 import mimetypes
 mimetypes.init()
@@ -21,6 +24,7 @@ from twisted.python import failure
 
 from coherence.upnp.core.DIDLLite import classChooser, Container, Resource
 from coherence.upnp.core.DIDLLite import DIDLElement
+from coherence.upnp.core.DIDLLite import simple_dlna_tags
 from coherence.upnp.core.soap_service import errorCode
 
 from coherence.upnp.core import utils
@@ -29,17 +33,17 @@ from coherence.extern.inotify import INotify
 from coherence.extern.inotify import IN_CREATE, IN_DELETE, IN_MOVED_FROM, IN_MOVED_TO, IN_ISDIR
 from coherence.extern.inotify import IN_CHANGED
 
+from coherence.extern.xdg import xdg_content
+
 import louie
 
-from coherence.extern.simple_plugin import Plugin
+from coherence.backend import BackendItem, BackendStore
 
-from coherence import log
-
-class FSItem(log.Loggable):
+class FSItem(BackendItem):
     logCategory = 'fs_item'
 
-    def __init__(self, id, parent, path, mimetype, urlbase, UPnPClass,update=False):
-        self.id = id
+    def __init__(self, object_id, parent, path, mimetype, urlbase, UPnPClass,update=False):
+        self.id = object_id
         self.parent = parent
         if parent:
             parent.add_child(self,update=update)
@@ -47,8 +51,9 @@ class FSItem(log.Loggable):
             self.location = unicode(path)
         else:
             if mimetype == 'item' and path is None:
-                path = os.path.join(parent.get_path(),str(self.id))
-            self.location = FilePath(unicode(path))
+                path = os.path.join(parent.get_path(),unicode(self.id))
+            #self.location = FilePath(unicode(path))
+            self.location = FilePath(path)
         self.mimetype = mimetype
         if urlbase[-1] != '/':
             urlbase += '/'
@@ -60,7 +65,7 @@ class FSItem(log.Loggable):
         else:
             parent_id = parent.get_id()
 
-        self.item = UPnPClass(id, parent_id, self.get_name())
+        self.item = UPnPClass(object_id, parent_id, self.get_name())
         if isinstance(self.item, Container):
             self.item.childCount = 0
         self.child_count = 0
@@ -69,22 +74,27 @@ class FSItem(log.Loggable):
 
         if mimetype in ['directory','root']:
             self.update_id = 0
+            self.get_url = lambda : self.url
+            self.get_path = lambda : None
             #self.item.searchable = True
             #self.item.searchClass = 'object'
-            self.check_for_cover_art()
-            if hasattr(self, 'cover'):
-                _,ext =  os.path.splitext(self.cover)
-                """ add the cover image extension to help clients not reacting on
-                    the mimetype """
-                self.item.albumArtURI = ''.join((urlbase,str(self.id),'?cover',ext))
+            if(isinstance(self.location,FilePath) and
+               self.location.isdir() == True):
+                self.check_for_cover_art()
+                if hasattr(self, 'cover'):
+                    _,ext =  os.path.splitext(self.cover)
+                    """ add the cover image extension to help clients not reacting on
+                        the mimetype """
+                    self.item.albumArtURI = ''.join((urlbase,str(self.id),'?cover',ext))
         else:
-            if hasattr(parent, 'cover'):
-                _,ext =  os.path.splitext(parent.cover)
-                """ add the cover image extension to help clients not reacting on
-                    the mimetype """
-                self.item.albumArtURI = ''.join((urlbase,str(self.id),'?cover',ext))
+            self.get_url = lambda : self.url
 
-            self.item.res = []
+            if self.mimetype.startswith('audio/'):
+                if hasattr(parent, 'cover'):
+                    _,ext =  os.path.splitext(parent.cover)
+                    """ add the cover image extension to help clients not reacting on
+                        the mimetype """
+                    self.item.albumArtURI = ''.join((urlbase,str(self.id),'?cover',ext))
 
             _,host_port,_,_,_ = urlsplit(urlbase)
             if host_port.find(':') != -1:
@@ -98,7 +108,7 @@ class FSItem(log.Loggable):
                 size = 0
 
             if mimetype != 'item':
-                res = Resource('file://'+self.get_path(), 'internal:%s:%s:*' % (host,self.mimetype))
+                res = Resource('file://'+ urllib2.quote(self.get_path()), 'internal:%s:%s:*' % (host,self.mimetype))
                 res.size = size
                 self.item.res.append(res)
 
@@ -110,6 +120,51 @@ class FSItem(log.Loggable):
             res.size = size
             self.item.res.append(res)
 
+
+            """ if this item is an image and we want to add a thumbnail for it
+                we have to follow these rules:
+
+                create a new Resource object, at least a 'http-get'
+                and maybe an 'internal' one too
+
+                for an JPG this looks like that
+
+                res = Resource(url_for_thumbnail,
+                        'http-get:*:image/jpg:%s'% ';'.join(simple_dlna_tags+('DLNA.ORG_PN=JPEG_TN',)))
+                res.size = size_of_thumbnail
+                self.item.res.append(res)
+
+                and for a PNG the Resource creation is like that
+
+                res = Resource(url_for_thumbnail,
+                        'http-get:*:image/png:%s'% ';'.join(simple_dlna_tags+('DLNA.ORG_PN=PNG_TN',)))
+
+                if not hasattr(self.item, 'attachments'):
+                    self.item.attachments = {}
+                self.item.attachments[key] = utils.StaticFile(filename_of_thumbnail)
+            """
+
+            if self.mimetype in ('image/jpeg', 'image/png'):
+                path = self.get_path()
+                thumbnail = os.path.join(os.path.dirname(path),'.thumbs',os.path.basename(path))
+                if os.path.exists(thumbnail):
+                    mimetype,_ = mimetypes.guess_type(thumbnail, strict=False)
+                    if mimetype in ('image/jpeg','image/png'):
+                        if mimetype == 'image/jpeg':
+                            dlna_pn = 'DLNA.ORG_PN=JPEG_TN'
+                        else:
+                            dlna_pn = 'DLNA.ORG_PN=PNG_TN'
+
+                        hash_from_path = str(id(thumbnail))
+                        new_res = Resource(self.url+'?attachment='+hash_from_path,
+                            'http-get:*:%s:%s' % (mimetype, ';'.join(simple_dlna_tags+(dlna_pn,))))
+                        new_res.size = os.path.getsize(thumbnail)
+                        self.item.res.append(new_res)
+                        if not hasattr(self.item, 'attachments'):
+                            self.item.attachments = {}
+                        self.item.attachments[hash_from_path] = utils.StaticFile(urllib2.quote(thumbnail))
+
+
             try:
                 # FIXME: getmtime is deprecated in Twisted 2.6
                 self.item.date = datetime.fromtimestamp(self.location.getmtime())
@@ -134,15 +189,13 @@ class FSItem(log.Loggable):
                 the mimetype """
             self.item.albumArtURI = ''.join((urlbase,str(self.id),'?cover',ext))
 
-        self.item.res = []
-
         _,host_port,_,_,_ = urlsplit(urlbase)
         if host_port.find(':') != -1:
             host,port = tuple(host_port.split(':'))
         else:
             host = host_port
 
-        res = Resource('file://'+self.get_path(), 'internal:%s:%s:*' % (host,self.mimetype))
+        res = Resource('file://'+urllib2.quote(self.get_path()), 'internal:%s:%s:*' % (host,self.mimetype))
         try:
             res.size = self.location.getsize()
         except:
@@ -173,15 +226,18 @@ class FSItem(log.Loggable):
             or png if the jpg search fails, and take the first one
             that comes around
         """
-        jpgs = [i.path for i in self.location.children() if i.splitext()[1] in ('.jpg', '.JPG')]
         try:
-            self.cover = jpgs[0]
-        except IndexError:
-            pngs = [i.path for i in self.location.children() if i.splitext()[1] in ('.png', '.PNG')]
+            jpgs = [i.path for i in self.location.children() if i.splitext()[1] in ('.jpg', '.JPG')]
             try:
-                self.cover = pngs[0]
+                self.cover = jpgs[0]
             except IndexError:
-                return
+                pngs = [i.path for i in self.location.children() if i.splitext()[1] in ('.png', '.PNG')]
+                try:
+                    self.cover = pngs[0]
+                except IndexError:
+                    return
+        except UnicodeDecodeError:
+            self.warning("UnicodeDecodeError - there is something wrong with a file located in %r", self.location.path)
 
     def remove(self):
         #print "FSItem remove", self.id, self.get_name(), self.parent
@@ -218,9 +274,6 @@ class FSItem(log.Loggable):
     def get_id(self):
         return self.id
 
-    def get_location(self):
-        return self.location
-
     def get_update_id(self):
         if hasattr(self, 'update_id'):
             return self.update_id
@@ -240,15 +293,15 @@ class FSItem(log.Loggable):
             path,old_ext = os.path.splitext(path)
             path = ''.join((path,extension))
         if isinstance( self.location,FilePath):
-            self.location = FilePath(unicode(path))
+            self.location = FilePath(path)
         else:
             self.location = path
 
     def get_name(self):
         if isinstance( self.location,FilePath):
-            name = self.location.basename()
+            name = self.location.basename().decode("utf-8", "replace")
         else:
-            name = self.location
+            name = self.location.decode("utf-8", "replace")
         return name
 
     def get_cover(self):
@@ -270,19 +323,29 @@ class FSItem(log.Loggable):
         return self.item.toString()
 
     def __repr__(self):
-        return 'id: ' + str(self.id) + ' @ ' + self.location.basename()
+        return 'id: ' + str(self.id) + ' @ ' + self.get_name().encode('ascii','xmlcharrefreplace')
 
-class FSStore(log.Loggable,Plugin):
+class FSStore(BackendStore):
     logCategory = 'fs_store'
 
     implements = ['MediaServer']
 
-    wmc_mapping = {'4':1000, '8':1000}
-
     def __init__(self, server, **kwargs):
+        BackendStore.__init__(self,server)
         self.next_id = 1000
         self.name = kwargs.get('name','my media')
-        self.content = kwargs.get('content','tests/content')
+        self.content = kwargs.get('content',None)
+        if self.content != None:
+                if isinstance(self.content,basestring):
+                    self.content = [self.content]
+                l = []
+                for a in self.content:
+                    l += a.split(',')
+                self.content = l
+        else:
+            self.content = xdg_content()
+        if self.content == None:
+            self.content = 'tests/content'
         if not isinstance( self.content, list):
             self.content = [self.content]
         self.urlbase = kwargs.get('urlbase','')
@@ -307,11 +370,18 @@ class FSStore(log.Loggable,Plugin):
             parent = self.store[id] = FSItem( id, parent, 'media', 'root', self.urlbase, UPnPClass, update=True)
 
         for path in self.content:
+            if isinstance(path,(list,tuple)):
+                path = path[0]
             if self.ignore_file_pattern.match(path):
                 continue
             self.walk(path, parent, self.ignore_file_pattern)
 
-        #self.update_id = 0
+        self.wmc_mapping.update({'14': 0,
+                                 '15': 0,
+                                 '16': 0,
+                                 '17': 0
+                                })
+
         louie.send('Coherence.UPnP.Backend.init_completed', None, backend=self)
 
     def __repr__(self):
@@ -321,6 +391,10 @@ class FSStore(log.Loggable,Plugin):
         return len(self.store)
 
     def get_by_id(self,id):
+        # we have referenced ids here when we are in WMC mapping mode
+        if isinstance(id, basestring):
+            id = id.split('@',1)
+            id = id[0]
         try:
             id = int(id)
         except ValueError:
@@ -337,6 +411,8 @@ class FSStore(log.Loggable,Plugin):
         try:
             parent = self.store[int(parent)]
             for child in parent.children:
+                if not isinstance(name, unicode):
+                    name = name.decode("utf8")
                 if name == child.get_name():
                     return child.id
         except:
@@ -351,14 +427,18 @@ class FSStore(log.Loggable,Plugin):
             containers.append(parent)
         while len(containers)>0:
             container = containers.pop()
-            for child in container.location.children():
-                if ignore_file_pattern.match(child.basename()) != None:
-                    continue
-                new_container = self.append(child.path,container)
-                if new_container != None:
-                    containers.append(new_container)
+            try:
+                for child in container.location.children():
+                    if ignore_file_pattern.match(child.basename()) != None:
+                        continue
+                    new_container = self.append(child.path,container)
+                    if new_container != None:
+                        containers.append(new_container)
+            except UnicodeDecodeError:
+                self.warning("UnicodeDecodeError - there is something wrong with a file located in %r", container.get_path())
 
     def create(self, mimetype, path, parent):
+        #print "create", mimetype, path, type(path)
         UPnPClass = classChooser(mimetype)
         if UPnPClass == None:
             return None
@@ -383,21 +463,25 @@ class FSStore(log.Loggable,Plugin):
         return id
 
     def append(self,path,parent):
-        #print "append", path
-        mimetype,_ = mimetypes.guess_type(path, strict=False)
-        if mimetype == None:
-            if os.path.isdir(path) or os.path.islink(path):
-                mimetype = 'directory'
-        if mimetype == None:
-            return None
+        #print "append", path, type(path)
+        try:
+            mimetype,_ = mimetypes.guess_type(path, strict=False)
+            if mimetype == None:
+                if os.path.isdir(path):
+                    mimetype = 'directory'
+            if mimetype == None:
+                return None
 
-        id = self.create(mimetype,path,parent)
+            id = self.create(mimetype,path,parent)
 
-        if mimetype == 'directory':
-            if self.inotify is not None:
-                mask = IN_CREATE | IN_DELETE | IN_MOVED_FROM | IN_MOVED_TO | IN_CHANGED
-                self.inotify.watch(path, mask=mask, auto_add=False, callbacks=(self.notify,id))
-            return self.store[id]
+            if mimetype == 'directory':
+                if self.inotify is not None:
+                    mask = IN_CREATE | IN_DELETE | IN_MOVED_FROM | IN_MOVED_TO | IN_CHANGED
+                    self.inotify.watch(path, mask=mask, auto_add=False, callbacks=(self.notify,id))
+                return self.store[id]
+        except OSError, msg:
+            """ seems we have some permissions issues along the content path """
+            self.warning("path %r isn't accessible, error %r", path, msg)
 
         return None
 
@@ -444,7 +528,7 @@ class FSStore(log.Loggable,Plugin):
             #else:
             #    print 'file %s was created, parent %d (%s)' % (path, parameter, iwp.path)
             if self.get_id_by_name(parameter,filename) is None:
-                if mask & IN_ISDIR:
+                if os.path.isdir(path):
                     self.walk(path, self.get_by_id(parameter), self.ignore_file_pattern)
                 else:
                     self.append(path, self.get_by_id(parameter))
@@ -526,7 +610,11 @@ class FSStore(log.Loggable,Plugin):
         return {'TransferID': transfer_id}
 
     def upnp_CreateObject(self, *args, **kwargs):
-        ContainerID = int(kwargs['ContainerID'])
+        if kwargs['ContainerID'] == 'DLNA.ORG_AnyContainer':
+            """ for the moment """
+            return failure.Failure(errorCode(712))
+        else:
+            ContainerID = kwargs['ContainerID']
         Elements = kwargs['Elements']
 
         parent_item = self.get_by_id(ContainerID)
