@@ -262,7 +262,7 @@ class Player(log.Loggable):
         if self.player.get_name() != 'player':
             self.create_pipeline(mimetype)
 
-        self.source.set_property(self.player_uri, uri)
+        self.source.set_property(self.player_uri, uri.encode('utf-8'))
         self.player_clean = True
         self.duration = None
         self.mimetype = mimetype
@@ -386,7 +386,8 @@ class GStreamerPlayer(log.Loggable,Plugin):
 
     logCategory = 'gstreamer_player'
     implements = ['MediaRenderer']
-    vendor_value_defaults = {'RenderingControl': {'A_ARG_TYPE_Channel':'Master'}}
+    vendor_value_defaults = {'RenderingControl': {'A_ARG_TYPE_Channel':'Master'},
+                             'AVTransport': {'A_ARG_TYPE_SeekMode':('ABS_TIME','REL_TIME')}}
     vendor_range_defaults = {'RenderingControl': {'Volume': {'maximum':100}}}
 
     def __init__(self, device, **kwargs):
@@ -484,10 +485,7 @@ class GStreamerPlayer(log.Loggable,Plugin):
         print "loading:", uri, mimetype
         _,state,_ = self.player.get_state()
         connection_id = self.server.connection_manager_server.lookup_avt_id(self.current_connection_id)
-        if( state == gst.STATE_PLAYING or state == gst.STATE_PAUSED):
-            self.stop()
-        else:
-            self.server.av_transport_server.set_variable(connection_id, 'TransportState', 'STOPPED')
+        self.stop(silent=True) # the check whether a stop is really needed is done inside stop
 
         if mimetype is None:
             _,ext =  os.path.splitext(uri)
@@ -508,13 +506,14 @@ class GStreamerPlayer(log.Loggable,Plugin):
 
         #self.server.av_transport_server.set_variable(connection_id, 'TransportState', 'TRANSITIONING')
         #self.server.av_transport_server.set_variable(connection_id, 'CurrentTransportActions','Play,Stop,Pause,Seek,Next,Previous')
-        self.server.av_transport_server.set_variable(connection_id, 'CurrentTransportActions','Play,Stop,Pause')
+        self.server.av_transport_server.set_variable(connection_id, 'CurrentTransportActions','Play,Stop,Pause,Seek')
         self.server.av_transport_server.set_variable(connection_id, 'NumberOfTracks',1)
         self.server.av_transport_server.set_variable(connection_id, 'CurrentTracks',1)
-        self.update()
         if state == gst.STATE_PLAYING:
             print "was playing..."
             self.play()
+        self.update()
+
 
     def status( self, position):
         uri = self.player.get_uri()
@@ -550,13 +549,14 @@ class GStreamerPlayer(log.Loggable,Plugin):
         self.load( uri)
         self.play()
 
-    def stop(self):
+    def stop(self,silent=False):
         print 'Stopping:', self.player.get_uri()
         if self.player.get_uri() == None:
             return
         if self.player.get_state()[1] in [gst.STATE_PLAYING,gst.STATE_PAUSED]:
             self.player.stop()
-            self.server.av_transport_server.set_variable(self.server.connection_manager_server.lookup_avt_id(self.current_connection_id), 'TransportState', 'STOPPED')
+            if silent is True:
+                self.server.av_transport_server.set_variable(self.server.connection_manager_server.lookup_avt_id(self.current_connection_id), 'TransportState', 'STOPPED')
 
     def play( self):
         print "Playing:", self.player.get_uri()
@@ -571,8 +571,9 @@ class GStreamerPlayer(log.Loggable,Plugin):
         self.player.pause()
         self.server.av_transport_server.set_variable(self.server.connection_manager_server.lookup_avt_id(self.current_connection_id), 'TransportState', 'PAUSED_PLAYBACK')
 
-    def seek(self, location):
+    def seek(self, location, old_state):
         self.player.seek(location)
+        self.server.av_transport_server.set_variable(0, 'TransportState', old_state)
 
     def mute(self):
         self.player.mute()
@@ -634,6 +635,18 @@ class GStreamerPlayer(log.Loggable,Plugin):
         self.stop()
         return {}
 
+    def upnp_Seek(self, *args, **kwargs):
+        InstanceID = int(kwargs['InstanceID'])
+        Unit = kwargs['Unit']
+        Target = kwargs['Target']
+        if Unit in ['ABS_TIME','REL_TIME']:
+            old_state = self.server.av_transport_server.get_variable(0, 'TransportState')
+            self.server.av_transport_server.set_variable(0, 'TransportState', 'TRANSITIONING')
+            h,m,s = Target.split(':')
+            seconds = int(h)*3600 + int(m)*60 + int(s)
+            self.seek(str(seconds), old_state)
+        return {}
+
     def upnp_SetAVTransportURI(self, *args, **kwargs):
         InstanceID = int(kwargs['InstanceID'])
         CurrentURI = kwargs['CurrentURI']
@@ -648,29 +661,14 @@ class GStreamerPlayer(log.Loggable,Plugin):
             elt = DIDLLite.DIDLElement.fromString(CurrentURIMetaData)
             if elt.numItems() == 1:
                 item = elt.getItems()[0]
-                for res in item.res:
-                    #print res.protocolInfo, res.data
-                    # FIXME:we can't rely on the sequence!
-                    #       if we accept internal:,
-                    #       we need to check _first_ if there
-                    #       are any matching ones,
-                    #       and if not try something else
-                    for protocol_info in local_protocol_infos:
-                        remote_protocol,remote_network,remote_content_format,_ = res.protocolInfo.split(':')
-                        print remote_protocol,remote_network,remote_content_format
-                        local_protocol,local_network,local_content_format,_ = protocol_info.split(':')
-                        print local_protocol,local_network,local_content_format
-                        if((remote_protocol == local_protocol or
-                            remote_protocol == '*' or
-                            local_protocol == '*') and
-                           (remote_network == local_network or
-                            remote_network == '*' or
-                            local_network == '*') and
-                           (remote_content_format == local_content_format or
-                            remote_content_format == '*' or
-                            local_content_format == '*')):
-                            self.load(res.data,CurrentURIMetaData,mimetype=remote_content_format)
-                            return {}
+                res = item.res.get_matching(local_protocol_infos, protocol_type='internal')
+                if len(res) == 0:
+                    res = item.res.get_matching(local_protocol_infos)
+                if len(res) > 0:
+                    res = res[0]
+                    remote_protocol,remote_network,remote_content_format,_ = res.protocolInfo.split(':')
+                    self.load(res.data,CurrentURIMetaData,mimetype=remote_content_format)
+                    return {}
         return failure.Failure(errorCode(714))
 
     def upnp_SetMute(self, *args, **kwargs):
