@@ -3,7 +3,7 @@
 # Licensed under the MIT license
 # http://opensource.org/licenses/mit-license.php
 
-# Copyright 2007, Frank Scholz <coherence@beebits.net>
+# Copyright 2007,2008, Frank Scholz <coherence@beebits.net>
 
 """
 
@@ -33,6 +33,11 @@ depends on:
 
                 pyid3lib - http://pyid3lib.sourceforge.net/doc.html
 
+                or
+
+                tagpy - http://news.tiker.net/software/tagpy
+                taglib - http://developer.kde.org/~wheeler/taglib.html
+
             CoversByAmazon - https://coherence.beebits.net/browser/trunk/coherence/extern/covers_by_amazon.py
 """
 
@@ -47,14 +52,34 @@ from epsilon.extime import Time
 
 import louie
 
+from twisted.internet import reactor, defer
+
 from twisted.python.filepath import FilePath
 from coherence.upnp.core import DIDLLite
 
 from coherence.extern.covers_by_amazon import CoverGetter
 
-from coherence import log
+from coherence.backend import BackendItem, BackendStore
 
-from coherence.extern.simple_plugin import Plugin
+
+def _dict_from_tags(tag):
+    tags = {}
+    tags['artist'] = tag.artist.strip()
+    tags['album'] = tag.album.strip()
+    tags['title'] = tag.title.strip()
+    if type(tag.track) == int:
+        tags['track'] = tag.track
+    elif type(tag.track) in (str, unicode):
+        tags['track'] = int(tag.track.strip())
+    else:
+        tags['track'] = tag.track[0]
+
+    for key in ('artist', 'album', 'title'):
+        value = tags.get(key, u'')
+        if isinstance(value, unicode):
+            tags[key] = value.encode('utf-8')
+
+    return tags
 
 try:
     import libmtag
@@ -74,28 +99,30 @@ except ImportError:
 
         def get_tags(filename):
             audio_file = pyid3lib.tag(filename)
-            tags = {}
-            try:
-                tags['artist'] = audio_file.artist.strip()
-                tags['album'] = audio_file.album.strip()
-                tags['title'] = audio_file.title.strip()
-                tags['track'] = audio_file.track[0]
-                return tags
-            except AttributeError,msg:
-                raise AttributeError
+            return _dict_from_tags(audio_file)
 
     except ImportError:
-        raise ImportError, "we need some installed id3 tag library for this backend"
+        try:
+            import tagpy
+
+            def get_tags(filename):
+                audio_file = tagpy.FileRef(filename)
+                return _dict_from_tags(audio_file.tag())
+        except ImportError:
+            get_tags = None
+
+if not get_tags:
+    raise ImportError, "we need some installed id3 tag library for this backend: python-tagpy, pyid3lib or libmtag"
 
 
 
 MEDIA_DB = 'tests/media.db'
 
 ROOT_CONTAINER_ID = 0
-AUDIO_CONTAINER = 10
-AUDIO_ALL_CONTAINER_ID = 11
-AUDIO_ARTIST_CONTAINER_ID = 12
-AUDIO_ALBUM_CONTAINER_ID = 13
+AUDIO_CONTAINER = 100
+AUDIO_ALL_CONTAINER_ID = 101
+AUDIO_ARTIST_CONTAINER_ID = 102
+AUDIO_ALBUM_CONTAINER_ID = 103
 
 def sanitize(filename):
     badchars = ''.join(set(string.punctuation) - set('-_+.~'))
@@ -105,7 +132,7 @@ def sanitize(filename):
     f = f.replace(badchars, '_')
     return f
 
-class Container(object):
+class Container(BackendItem):
 
     def __init__(self, id, parent_id, name, children_callback=None):
         self.id = id
@@ -149,7 +176,7 @@ class Container(object):
     def get_id(self):
         return self.id
 
-class Artist(item.Item):
+class Artist(item.Item,BackendItem):
     """ definition for an artist """
 
     schemaVersion = 1
@@ -180,6 +207,7 @@ class Artist(item.Item):
 
     def get_item(self):
         item = DIDLLite.MusicArtist(self.storeID+1000, AUDIO_ARTIST_CONTAINER_ID, self.name)
+        item.childCount = self.get_child_count()
         return item
 
     def get_id(self):
@@ -192,7 +220,7 @@ class Artist(item.Item):
         return '<Artist %d name="%s" musicbrainz="%s">' \
                % (self.storeID, self.name.encode('ascii', 'ignore'), self.musicbrainz_id)
 
-class Album(item.Item):
+class Album(item.Item,BackendItem):
     """ definition for an album """
 
     schemaVersion = 1
@@ -217,6 +245,8 @@ class Album(item.Item):
 
     def get_item(self):
         item = DIDLLite.MusicAlbum(self.storeID+1000, AUDIO_ALBUM_CONTAINER_ID, self.title)
+        item.artist = self.artist.name
+        item.childCount = self.get_child_count()
         if len(self.cover)>0:
             _,ext =  os.path.splitext(self.cover)
             item.albumArtURI = ''.join((self.store.urlbase,str(self.get_id()),'?cover',ext))
@@ -239,7 +269,7 @@ class Album(item.Item):
                   self.cover.encode('ascii', 'ignore'),
                   self.musicbrainz_id)
 
-class Track(item.Item):
+class Track(item.Item,BackendItem):
     """ definition for a track """
 
     schemaVersion = 1
@@ -269,7 +299,8 @@ class Track(item.Item):
             """ add the cover image extension to help clients not reacting on
                 the mimetype """
             item.albumArtURI = ''.join((self.store.urlbase,str(self.storeID+1000),'?cover',ext))
-        item.res = []
+        item.originalTrackNumber = self.track_nr
+        item.server_uuid = str(self.store.server.uuid)[5:]
 
         _,host_port,_,_,_ = urlsplit(self.store.urlbase)
         if host_port.find(':') != -1:
@@ -332,7 +363,7 @@ class Track(item.Item):
                   self.album.artist.name.encode('ascii', 'ignore'),
                   self.location.encode('ascii', 'ignore'))
 
-class Playlist(item.Item):
+class Playlist(item.Item,BackendItem):
     """ definition for a playlist
 
         - has a name
@@ -348,7 +379,7 @@ class Playlist(item.Item):
     name = attributes.text(allowNone=False, indexed=True)
     # references to tracks
 
-class MediaStore(log.Loggable, Plugin):
+class MediaStore(BackendStore):
     logCategory = 'media_store'
     implements = ['MediaServer']
 
@@ -374,6 +405,12 @@ class MediaStore(log.Loggable, Plugin):
         self.containers[ROOT_CONTAINER_ID] = \
                 Container( ROOT_CONTAINER_ID,-1, self.name)
 
+        self.wmc_mapping.update({'4': lambda : self.get_by_id(AUDIO_ALL_CONTAINER_ID),    # all tracks
+                                 '7': lambda : self.get_by_id(AUDIO_ALBUM_CONTAINER_ID),    # all albums
+                                 '6': lambda : self.get_by_id(AUDIO_ARTIST_CONTAINER_ID),    # all artists
+                                })
+
+
         louie.send('Coherence.UPnP.Backend.init_completed', None, backend=self)
 
     def walk(self, path):
@@ -410,21 +447,22 @@ class MediaStore(log.Loggable, Plugin):
                 except IndexError:
                     return u''
 
-        for file in self.filelist:
-            tags = get_tags(file)
+        def got_tags(tags, file):
+            #print "got_tags", tags
+
             album=tags.get('album', '')
             artist=tags.get('artist', '')
             title=tags.get('title', '')
             track=tags.get('track', 0)
 
             if len(artist) == 0:
-                continue;
+                return;
                 artist = u'UNKNOWN_ARTIST'
             if len(album) == 0:
-                continue;
+                return;
                 album = u'UNKNOWN_ALBUM'
             if len(title) == 0:
-                continue;
+                return;
                 title = u'UNKNOWN_TITLE'
 
             #print "Tags:", file, album, artist, title, track
@@ -449,6 +487,11 @@ class MediaStore(log.Loggable, Plugin):
                                             album=album_ds,
                                             location=unicode(file,'utf8'))
 
+        for file in self.filelist:
+            d = defer.maybeDeferred(get_tags,file)
+            d.addBoth(got_tags, file)
+
+
     def show_db(self):
         for album in list(self.db.query(Album,sort=Album.title.ascending)):
             print album
@@ -516,7 +559,10 @@ class MediaStore(log.Loggable, Plugin):
 
     def get_by_id(self,id):
         self.info("get_by_id %s" % id)
-        if id.startswith('artist_all_tracks_'):
+        if isinstance(id, basestring):
+            id = id.split('@',1)
+            id = id[0]
+        if isinstance(id, basestring) and id.startswith('artist_all_tracks_'):
             try:
                 return self.containers[id]
             except:
@@ -555,6 +601,7 @@ class MediaStore(log.Loggable, Plugin):
                           children_callback=lambda :list(self.db.query(Artist,sort=Artist.name.ascending)))
         self.containers[ROOT_CONTAINER_ID].add_child(self.containers[AUDIO_ARTIST_CONTAINER_ID])
 
+        self.db.server = self.server
         self.db.urlbase = self.urlbase
         self.db.containers = self.containers
 
