@@ -24,6 +24,73 @@ from coherence.upnp.core import utils
 
 from coherence.upnp.core import dlna
 
+from coherence import log
+
+class Resources(list):
+
+    """ a list of resources, always sorted after an append """
+
+    def __init__(self, *args, **kwargs):
+        list.__init__(self, *args, **kwargs)
+        self.sort(cmp=self.p_sort)
+
+    def append(self, value):
+        list.append(self,value)
+        self.sort(cmp=self.p_sort)
+
+    def p_sort(self,x,y):
+        """ we want the following order
+            http-get is always at the beginning
+            rtsp-rtp-udp the second
+            anything else after that
+        """
+        if x.protocolInfo == None:
+            return 1
+        if y.protocolInfo == None:
+            return -1
+
+        x_protocol = x.protocolInfo.split(':')[0]
+        y_protocol = y.protocolInfo.split(':')[0]
+
+        x_protocol = x_protocol.lower()
+        y_protocol = y_protocol.lower()
+        if( x_protocol == y_protocol):
+            return 0
+        if(x_protocol == 'http-get'):
+            return -1
+        if(x_protocol == 'rtsp-rtp-udp' and y_protocol == 'http-get'):
+            return 1
+        if(x_protocol == 'rtsp-rtp-udp' and y_protocol != 'http-get'):
+            return -1
+        return 1
+
+    def get_matching(self, local_protocol_infos, protocol_type = None):
+        result = []
+        if not isinstance(local_protocol_infos, list):
+            local_protocol_infos = [local_protocol_infos]
+        for res in self:
+            #print "res", res.protocolInfo, res.data
+            remote_protocol,remote_network,remote_content_format,_ = res.protocolInfo.split(':')
+            #print "remote", remote_protocol,remote_network,remote_content_format
+            if(protocol_type is not None and
+               remote_protocol.lower() != protocol_type.lower()):
+                continue
+            for protocol_info in local_protocol_infos:
+                local_protocol,local_network,local_content_format,_ = protocol_info.split(':')
+                #print "local", local_protocol,local_network,local_content_format
+                if((remote_protocol == local_protocol or
+                    remote_protocol == '*' or
+                    local_protocol == '*') and
+                   (remote_network == local_network or
+                    remote_network == '*' or
+                    local_network == '*') and
+                   (remote_content_format == local_content_format or
+                    remote_content_format == '*' or
+                    local_content_format == '*')):
+                        #print result, res
+                        result.append(res)
+        return result
+
 def classChooser(mimetype, sub=None):
 
     if mimetype == 'root':
@@ -49,6 +116,11 @@ def classChooser(mimetype, sub=None):
             return AudioItem
     return None
 
+simple_dlna_tags = ('DLNA.ORG.PS=1',       # play speed parameter
+                    'DLNA.ORG_CI=0',       # transcoded parameter
+                    'DLNA.ORG_OP=01',      # operations parameter
+                    'DLNA.ORG_FLAGS=01700000000000000000000000000000')
+
 
 class Resource:
     """An object representing a resource."""
@@ -66,15 +138,17 @@ class Resource:
             protocol,network,content_format,additional_info = self.protocolInfo.split(':')
             if additional_info == '*':
                 if content_format == 'audio/mpeg':
-                    additional_info = ';'.join(('DLNA.ORG_PN=MP3','DLNA.ORG_OP=01'))
+                    additional_info = ';'.join(simple_dlna_tags+('DLNA.ORG_PN=MP3',))
                 if content_format == 'image/jpeg':
-                    additional_info = ';'.join(('DLNA.ORG_PN=JPEG_SM','DLNA.ORG_OP=01'))
+                    additional_info = ';'.join(simple_dlna_tags+('DLNA.ORG_PN=JPEG_LRG',))
+                if content_format == 'image/png':
+                    additional_info = ';'.join(simple_dlna_tags+('DLNA.ORG_PN=PNG_LRG',))
                 if content_format == 'video/mpeg':
-                    additional_info = ';'.join(('DLNA.ORG_PN=MPEG_PS_PAL','DLNA.ORG_OP=01'))
+                    additional_info = ';'.join(simple_dlna_tags+('DLNA.ORG_PN=MPEG_PS_PAL',))
                 if content_format == 'video/mp4':
-                    additional_info = ';'.join(('DLNA.ORG_PN=AVC_TS_BL_CIF15_AAC','DLNA.ORG_OP=01'))
+                    additional_info = ';'.join(simple_dlna_tags+('DLNA.ORG_PN=AVC_TS_BL_CIF15_AAC',))
                 if content_format == 'video/x-msvideo':
-                    additional_info = ';'.join(('DLNA.ORG_PN=MPEG4_P2_MP4_SP_AAC','DLNA.ORG_OP=01'))
+                    additional_info = ';'.join(simple_dlna_tags+('DLNA.ORG_PN=MPEG4_P2_MP4_SP_AAC',))
 
                 self.protocolInfo = ':'.join((protocol,network,content_format,additional_info))
 
@@ -85,6 +159,11 @@ class Resource:
             protocol,network,content_format,additional_info = self.protocolInfo.split(':')
             if content_format == 'video/x-msvideo':
                 content_format = 'video/avi'
+            if kwargs.get('upnp_client','') == 'XBox':
+                """ we don't need the DLNA tags there,
+                    and maybe it irritates that poor thing anyway
+                """
+                additional_info = '*'
             root.attrib['protocolInfo'] = ':'.join((protocol,network,content_format,additional_info))
         else:
             root.attrib['protocolInfo'] = self.protocolInfo
@@ -123,9 +202,11 @@ class Resource:
         instance.fromElement(elt.getroot())
         return instance
 
-class Object:
+class Object(log.Loggable):
     """The root class of the entire content directory class heirachy."""
 
+    logCategory = 'didllite'
+
     upnp_class = 'object'
     creator = None
     #res = None
@@ -133,11 +214,14 @@ class Object:
     date = None
     albumArtURI = None
     artist = None
+    album = None
     originalTrackNumber=None
 
+    refID = None
+    server_uuid = None
+
     def __init__(self, id=None, parentID=None, title=None, restricted=False,
                        creator=None):
-        self.res = []
         self.id = id
         self.parentID = parentID
         self.title = title
@@ -160,6 +244,32 @@ class Object:
 
         root.attrib['parentID'] = str(self.parentID)
 
+        if(kwargs.get('upnp_client','') != 'XBox'):
+            if self.refID:
+                root.attrib['refID'] = str(self.refID)
+
+        if kwargs.get('requested_id',None):
+            if kwargs.get('requested_id') != root.attrib['id']:
+                if(kwargs.get('upnp_client','') != 'XBox'):
+                    root.attrib['refID'] = root.attrib['id']
+                r_id = kwargs.get('requested_id')
+                root.attrib['id'] = r_id
+                r_id = r_id.split('@',1)
+                try:
+                    root.attrib['parentID'] = r_id[1]
+                except IndexError:
+                    pass
+                self.info("Changing ID from %r to %r, with parentID %r", root.attrib['refID'], root.attrib['id'], root.attrib['parentID'])
+        elif kwargs.get('parent_container',None):
+            if(kwargs.get('parent_container') != '0' and
+               kwargs.get('parent_container') != root.attrib['parentID']):
+                if(kwargs.get('upnp_client','') != 'XBox'):
+                    root.attrib['refID'] = root.attrib['id']
+                root.attrib['id'] = '@'.join((root.attrib['id'],kwargs.get('parent_container')))
+                root.attrib['parentID'] = kwargs.get('parent_container')
+                self.info("Changing ID from %r to %r, with parentID from %r to %r", root.attrib['refID'], root.attrib['id'], root.attrib['parentID'],kwargs.get('parent_container'))
+
+
         if(isinstance(self, Container) and kwargs.get('upnp_client','') == 'XBox'):
             ET.SubElement(root, 'upnp:class').text = 'object.container.storageFolder'
         else:
@@ -173,9 +283,6 @@ class Object:
         if self.creator is not None:
             ET.SubElement(root, 'dc:creator').text = self.creator
 
-        for res in self.res:
-            root.append(res.toElement(**kwargs))
-
         if self.writeStatus is not None:
             ET.SubElement(root, 'upnp:writeStatus').text = self.writeStatus
 
@@ -199,6 +306,9 @@ class Object:
         if self.originalTrackNumber is not None:
             ET.SubElement(root, 'upnp:originalTrackNumber').text = str(self.originalTrackNumber)
 
+        if self.server_uuid is not None:
+            ET.SubElement(root, 'upnp:server_uuid').text = self.server_uuid
+
         return root
 
     def toString(self,**kwargs):
@@ -211,9 +321,12 @@ class Object:
          * writeStatus
         """
         self.elementName = elt.tag
-        self.id = elt.attrib['id']
-        self.parentID = elt.attrib['parentID']
-        if elt.attrib['restricted'] in [1,'true','True','1','yes','Yes']:
+        self.id = elt.attrib.get('id',None)
+        self.parentID = elt.attrib.get('parentID',None)
+
+        self.refID = elt.attrib.get('refID',None)
+
+        if elt.attrib.get('restricted',None) in [1,'true','True','1','yes','Yes']:
             self.restricted = True
         else:
             self.restricted = False
@@ -221,17 +334,18 @@ class Object:
         for child in elt.getchildren():
             if child.tag.endswith('title'):
                 self.title = child.text
-            if child.tag.endswith('albumArtURI'):
+            elif child.tag.endswith('albumArtURI'):
                 self.albumArtURI = child.text
-            if child.tag.endswith('originalTrackNumber'):
-                self.originalTrackNumber = child.text
-            if child.tag.endswith('artist'):
+            elif child.tag.endswith('originalTrackNumber'):
+                self.originalTrackNumber = int(child.text)
+            elif child.tag.endswith('artist'):
                 self.artist = child.text
+            elif child.tag.endswith('album'):
+                self.album = child.text
             elif child.tag.endswith('class'):
                 self.upnp_class = child.text
-            elif child.tag.endswith('res'):
-                res = Resource.fromString(ET.tostring(child))
-                self.res.append(res)
+            elif child.tag.endswith('server_uuid'):
+                self.server_uuid = child.text
 
 
     @classmethod
@@ -251,6 +365,10 @@ class Item(Object):
     elementName = 'item'
     refID = None
 
+    def __init__(self, *args, **kwargs):
+        Object.__init__(self, *args, **kwargs)
+        self.res = Resources()
+
     def toElement(self,**kwargs):
 
         root = Object.toElement(self,**kwargs)
@@ -258,6 +376,9 @@ class Item(Object):
         if self.refID is not None:
             ET.SubElement(root, 'refID').text = self.refID
 
+        for res in self.res:
+            root.append(res.toElement(**kwargs))
+
         return root
 
     def fromElement(self, elt):
@@ -265,8 +386,9 @@ class Item(Object):
         for child in elt.getchildren():
             if child.tag.endswith('refID'):
                 self.refID = child.text
-                break
-
+            elif child.tag.endswith('res'):
+                res = Resource.fromString(ET.tostring(child))
+                self.res.append(res)
 
 class ImageItem(Item):
     upnp_class = Item.upnp_class + '.imageItem'
@@ -384,7 +506,7 @@ class MusicTrack(AudioItem):
 
         if self.originalTrackNumber is not None:
             ET.SubElement(root, 'upnp:originalTrackNumber').text = \
-                             self.originalTrackNumber
+                             str(self.originalTrackNumber)
 
         if self.playlist is not None:
             ET.SubElement(root, 'upnp:playlist').text = self.playlist
@@ -512,9 +634,11 @@ class StorageVolume(Container):
 class StorageFolder(Container):
     upnp_class = Container.upnp_class + '.storageFolder'
 
-class DIDLElement(ElementInterface):
+class DIDLElement(ElementInterface,log.Loggable):
 
-    def __init__(self, upnp_client=''):
+    logCategory = 'didllite'
+
+    def __init__(self, upnp_client='', parent_container=None, requested_id=None):
         ElementInterface.__init__(self, 'DIDL-Lite', {})
         self.attrib['xmlns'] = 'urn:schemas-upnp-org:metadata-1-0/DIDL-Lite/'
         self.attrib['xmlns:dc'] = 'http://purl.org/dc/elements/1.1/'
@@ -523,13 +647,18 @@ class DIDLElement(ElementInterface):
         self.attrib['xmlns:pv'] = 'http://www.pv.com/pvns/'
         self._items = []
         self.upnp_client = upnp_client
+        self.parent_container = parent_container
+        self.requested_id = requested_id
+
 
     def addContainer(self, id, parentID, title, restricted = False):
         e = Container(id, parentID, title, restricted, creator = '')
         self.append(e.toElement())
 
     def addItem(self, item):
-        self.append(item.toElement(upnp_client=self.upnp_client))
+        self.append(item.toElement(upnp_client=self.upnp_client,
+                                   parent_container=self.parent_container,
+                                   requested_id=self.requested_id))
         self._items.append(item)
 
     def numItems(self):
@@ -546,6 +675,22 @@ class DIDLElement(ElementInterface):
         #return preamble + ET.tostring(self,encoding='utf-8')
         return ET.tostring(self,encoding='utf-8')
 
+    def get_upnp_class(self,name):
+        try:
+            return upnp_classes[name]()
+        except KeyError:
+            self.warning("upnp_class %r not found, trying fallback", name)
+            parts = name.split('.')
+            parts.pop()
+            while len(parts) > 1:
+                try:
+                    return upnp_classes['.'.join(parts)]()
+                except KeyError:
+                    parts.pop()
+
+        self.warning("WTF - no fallback for upnp_class %r found ?!?", name)
+        return None
+
     @classmethod
     def fromString(cls, aString):
         instance = cls()
@@ -553,10 +698,7 @@ class DIDLElement(ElementInterface):
         elt = elt.getroot()
         for node in elt.getchildren():
             upnp_class_name =  node.findtext('{%s}class' % 'urn:schemas-upnp-org:metadata-1-0/upnp/')
-            try:
-                upnp_class = upnp_classes[upnp_class_name]()
-            except Exception, msg:
-                print msg
+            upnp_class = instance.get_upnp_class(upnp_class_name)
             new_node = upnp_class.fromString(ET.tostring(node))
             instance.addItem(new_node)
         return instance
@@ -589,3 +731,18 @@ upnp_classes = {'object': Object,
                 'object.container.storageVolume': StorageVolume,
                 'object.container.storageFolder': StorageFolder,
 }
+
+
+if __name__ == '__main__':
+
+    res = Resources()
+    res.append(Resource('1','file:*:*:*'))
+    res.append(Resource('2','rtsp-rtp-udp:*:*:*'))
+    res.append(Resource('3',None))
+    res.append(Resource('4','internal:*:*:*'))
+    res.append(Resource('5','http-get:*:*:*'))
+    res.append(Resource('6','something:*:*:*'))
+    res.append(Resource('7','http-get:*:*:*'))
+
+    for r in res:
+        print r.data, r.protocolInfo
\ No newline at end of file
