@@ -4,6 +4,8 @@
 # Copyright (C) 2006 Fluendo, S.A. (www.fluendo.com).
 # Copyright 2006, Frank Scholz <coherence@beebits.net>
 
+import urlparse
+
 from coherence.extern.et import parse_xml as et_parse_xml
 
 from twisted.web import server, http, static
@@ -12,19 +14,19 @@ from twisted.web import proxy, resource, server
 from twisted.internet import reactor, protocol
 from twisted.python import failure
 
-import socket
-import fcntl
-import struct
-import string
-import os
-import urlparse
+try:
+    import netifaces
+    have_netifaces = True
+except ImportError:
+    have_netifaces = False
 
 def parse_xml(data, encoding="utf-8"):
     return et_parse_xml(data,encoding)
 
 def parse_http_response(data):
 
-    header, payload = data.split('\r\n\r\n')
+    """ don't try to get the body, there are reponses without """
+    header = data.split('\r\n\r\n')[0]
 
     lines = header.split('\r\n')
     cmd = lines[0].split(' ')
@@ -49,9 +51,31 @@ def get_ip_address(ifname):
 
     Updated to work on BSD. OpenBSD and OSX share the same value for
     SIOCGIFADDR, and its likely that other BSDs do too.
+
+    Updated to work on Windows,
+    using the optional Python module netifaces
+    http://alastairs-place.net/netifaces/
+
+    Thx Lawrence for that patch!
     """
 
-    system_type = os.uname()[0]
+    import sys
+    if sys.platform == 'win32':
+        if have_netifaces:
+            if ifname in netifaces.interfaces():
+                iface = netifaces.ifaddresses(ifname)
+                ifaceadr = iface(netifaces.AF_INET)
+                # we now have a list of address dictionaries, there may be multiple addresses bound
+                return ifaceadr[0]['addr']
+        return '127.0.0.1'
+
+    from os import uname
+    import socket
+    import fcntl
+    import struct
+
+    system_type = uname()[0]
+
     if system_type == "Linux":
         SIOCGIFADDR = 0x8915
     else:
@@ -69,6 +93,12 @@ def get_host_address():
         the default route, as this is most likely
         the interface we should bind to (on a single homed host!)
     """
+
+    import sys
+    if sys.platform == 'win32':
+        if have_netifaces:
+            return get_ip_address(netifaces.interfaces()[0])    # on windows assume first interface is primary
+
     try:
         route_file = '/proc/net/route'
         route = open(route_file)
@@ -83,8 +113,9 @@ def get_host_address():
                         return get_ip_address(l[0])
     except IOerror:
         """ fallback to parsing the output of netstat """
-        import os, posix
-        (osname,_, _, _,_) = os.uname()
+        from os import uname
+        import posix
+        (osname,_, _, _,_) = uname()
         osname = osname.lower()
         f = posix.popen('netstat -rn')
         lines = f.readlines()
@@ -97,9 +128,35 @@ def get_host_address():
                 else:
                     return get_ip_address(parts[-1])
 
-    """ return localhost if we havn't found anything """
+    """ return localhost if we haven't found anything """
     return '127.0.0.1'
 
+def de_chunk_payload(response):
+
+    import StringIO
+    """ This method takes a chunked HTTP data object and unchunks it."""
+    newresponse = StringIO.StringIO()
+    # chunked encoding consists of a bunch of lines with
+    # a length in hex followed by a data chunk and a CRLF pair.
+    response = StringIO.StringIO(response)
+
+    def read_chunk_length():
+        line = response.readline()
+        try:
+            len = int(line.strip(),16)
+        except ValueError:
+            len = 0
+        return len
+
+    len = read_chunk_length()
+    while (len > 0):
+        newresponse.write(response.read(len))
+        line = response.readline() # after chunk and before next chunk length
+        len = read_chunk_length()
+
+    return newresponse.getvalue()
+
+
 class Site(server.Site):
 
     noisy = False
@@ -117,14 +174,13 @@ class ProxyClient(http.HTTPClient):
         self.rest = rest
         if headers.has_key("proxy-connection"):
             del headers["proxy-connection"]
-        #headers["connection"] = "close"
+        headers["connection"] = "close"
         self.headers = headers
-        #if not headers.has_key("keep-alive"):
-        #    headers["keep-alive"] = ''
         print "command", command
         print "rest", rest
         print "headers", headers
         self.data = data
+        self.send_data = 0
 
     def connectionMade(self):
         self.sendCommand(self.command, self.rest)
@@ -134,6 +190,11 @@ class ProxyClient(http.HTTPClient):
         self.transport.write(self.data)
 
     def handleStatus(self, version, code, message):
+        if message:
+            # Add a whitespace to message, this allows empty messages
+            # transparently
+            message = " %s" % (message,)
+
         if version == 'ICY':
             version = 'HTTP/1.1'
         print "ProxyClient handleStatus", version, code, message
@@ -155,22 +216,22 @@ class ProxyClient(http.HTTPClient):
         self.father.transport.write("\r\n")
 
     def handleResponsePart(self, buffer):
-        #print "ProxyClient hanhttp://twistedmatrix.com/trac/ticket/1089dleResponsePart", len(buffer)
-        self.father.transport.write(buffer)
+        #print "ProxyClient handleResponsePart", len(buffer), self.father.chunked
+        self.send_data += len(buffer)
+        self.father.write(buffer)
 
     def handleResponseEnd(self):
+        #print "handleResponseEnd", self.send_data
         self.transport.loseConnection()
         self.father.channel.transport.loseConnection()
 
 
 class ProxyClientFactory(protocol.ClientFactory):
-    """ Used by ProxyRequest to implement a simple web proxy."""
-    """ Taken from twisted.web.proxy, to get around          """
-    """ http://twistedmatrix.com/trac/ticket/1089            """
-    """ until that's in the mainstream                       """
+    """
+    Used by ProxyRequest to implement a simple web proxy.
+    """
 
-    #protocol = proxy.ProxyClient
-    protocol = ProxyClient
+    protocol = proxy.ProxyClient
 
 
     def __init__(self, command, rest, version, headers, data, father):
@@ -196,36 +257,69 @@ class ProxyClientFactory(protocol.ClientFactory):
 
 
 class ReverseProxyResource(proxy.ReverseProxyResource):
-    """Resource that renders the results gotten from another server
+    """
+    Resource that renders the results gotten from another server
 
     Put this resource in the tree to cause everything below it to be relayed
     to a different server.
+
+    @ivar proxyClientFactoryClass: a proxy client factory class, used to create
+        new connections.
+    @type proxyClientFactoryClass: L{ClientFactory}
+
+    @ivar reactor: the reactor used to create connections.
+    @type reactor: object providing L{twisted.internet.interfaces.IReactorTCP}
     """
 
-    def __init__(self, host, port, path):
+    proxyClientFactoryClass = ProxyClientFactory
+
+
+    def __init__(self, host, port, path, reactor=reactor):
+        """
+        @param host: the host of the web server to proxy.
+        @type host: C{str}
+
+        @param port: the port of the web server to proxy.
+        @type port: C{port}
+
+        @param path: the base path to fetch data from. Note that you shouldn't
+            put any trailing slashes in it, it will be added automatically in
+            request. For example, if you put B{/foo}, a request on B{/bar} will
+            be proxied to B{/foo/bar}.
+        @type path: C{str}
+        """
         resource.Resource.__init__(self)
         self.host = host
         self.port = port
         self.path = path
-        self.connection = None
+        self.reactor = reactor
+
 
     def getChild(self, path, request):
-        return ReverseProxyResource(self.host, self.port, self.path+'/'+path)
+        return ReverseProxyResource(
+            self.host, self.port, self.path + '/' + path)
+
 
     def render(self, request):
-        request.received_headers['host'] = self.host
+        """
+        Render a request by forwarding it to the proxied server.
+        """
+        # RFC 2616 tells us that we can omit the port if it's the default port,
+        # but we have to provide it otherwise
+        if self.port == 80:
+            request.received_headers['host'] = self.host
+        else:
+            request.received_headers['host'] = "%s:%d" % (self.host, self.port)
         request.content.seek(0, 0)
         qs = urlparse.urlparse(request.uri)[4]
         if qs:
             rest = self.path + '?' + qs
         else:
             rest = self.path
-        clientFactory = ProxyClientFactory(request.method, rest,
-                                     request.clientproto,
-                                     request.getAllHeaders(),
-                                     request.content.read(),
-                                     request)
-        self.connection = reactor.connectTCP(self.host, self.port, clientFactory)
+        clientFactory = self.proxyClientFactoryClass(
+            request.method, rest, request.clientproto,
+            request.getAllHeaders(), request.content.read(), request)
+        self.reactor.connectTCP(self.host, self.port, clientFactory)
         return server.NOT_DONE_YET
 
 class myHTTPPageGetter(client.HTTPPageGetter):
@@ -242,7 +336,11 @@ class myHTTPPageGetter(client.HTTPPageGetter):
             self.factory.noPage(failure.Failure(
                 client.PartialDownloadError(self.status, self.message, response)))
         else:
-            self.factory.page(response)
+            if(self.headers.has_key('transfer-encoding') and
+               self.headers['transfer-encoding'][0].lower() == 'chunked'):
+                self.factory.page(de_chunk_payload(response))
+            else:
+                self.factory.page(response)
         # server might be stupid and not close connection. admittedly
         # the fact we do only one request per connection is also
         # stupid...
@@ -319,7 +417,9 @@ class StaticFile(static.File):
     """
 
     def render(self, request):
+        #print ""
         #print "StaticFile", request
+        #print "StaticFile in", request.received_headers
 
         """You know what you doing."""
         self.restat()
@@ -366,10 +466,10 @@ class StaticFile(static.File):
         tsize = size
         if range is not None:
             # This is a request for partial data...
-            bytesrange = string.split(range, '=')
+            bytesrange = range.split('=')
             assert bytesrange[0] == 'bytes',\
                    "Syntactically invalid http range header!"
-            start, end = string.split(bytesrange[1],'-', 1)
+            start, end = bytesrange[1].split('-', 1)
             if start:
                 f.seek(int(start))
                 if end:
@@ -401,12 +501,15 @@ class StaticFile(static.File):
                 #print "StaticFile", start, end, tsize
 
         request.setHeader('content-length', str(fsize))
+
         if request.method == 'HEAD' or trans == False:
             # pretend we're a HEAD request, so content-length
             # won't be overwritten.
             request.method = 'HEAD'
             return ''
 
+        #print "StaticFile out", request.headers, request.code
+
         # return data
         # size is the byte position to stop sending, not how many bytes to send
         static.FileTransfer(f, size, request)
