@@ -32,11 +32,15 @@ from coherence.upnp.services.servers.content_directory_server import ContentDire
 from coherence.upnp.services.servers.media_receiver_registrar_server import MediaReceiverRegistrarServer
 from coherence.upnp.services.servers.media_receiver_registrar_server import FakeMediaReceiverRegistrarBackend
 
+from coherence.upnp.devices.basics import BasicDeviceMixin
+
 import louie
 
 from coherence import log
 
-COVER_REQUEST_INDICATOR = re.compile(".*cover\.[A-Z|a-z]{3,4}$")
+COVER_REQUEST_INDICATOR = re.compile(".*?cover\.[A-Z|a-z]{3,4}$")
+
+ATTACHMENT_REQUEST_INDICATOR = re.compile(".*?attachment=.*$")
 
 class MSRoot(resource.Resource, log.Loggable):
     logCategory = 'mediaserver'
@@ -53,18 +57,27 @@ class MSRoot(resource.Resource, log.Loggable):
         headers = request.getAllHeaders()
         self.msg( request.getAllHeaders())
 
-        if(request.method == 'GET' and
-           COVER_REQUEST_INDICATOR.match(request.uri)):
-            self.info("request cover for id %s" % path)
-            ch = self.store.get_by_id(path)
-            if ch is not None:
-                request.setResponseCode(200)
-                file = ch.get_cover()
-                if os.path.exists(file):
-                    self.info("got cover %s" % file)
-                    return StaticFile(file)
-            request.setResponseCode(404)
-            return static.Data('<html><p>cover requested not found</p></html>','text/html')
+        if request.method == 'GET':
+            if COVER_REQUEST_INDICATOR.match(request.uri):
+                self.info("request cover for id %s" % path)
+                ch = self.store.get_by_id(path)
+                if ch is not None:
+                    request.setResponseCode(200)
+                    file = ch.get_cover()
+                    if os.path.exists(file):
+                        self.info("got cover %s" % file)
+                        return StaticFile(file)
+                request.setResponseCode(404)
+                return static.Data('<html><p>cover requested not found</p></html>','text/html')
+
+            if ATTACHMENT_REQUEST_INDICATOR.match(request.uri):
+                self.info("request attachment %r for id %s" % (request.args,path))
+                ch = self.store.get_by_id(path)
+                try:
+                    return ch.item.attachments[request.args['attachment'][0]]
+                except:
+                    request.setResponseCode(404)
+                    return static.Data('<html><p>the requested attachment was not found</p></html>','text/html')
 
         if(request.method == 'POST' and
            request.uri.endswith('?import')):
@@ -85,8 +98,10 @@ class MSRoot(resource.Resource, log.Loggable):
             return self
         return self.getChild(path, request)
 
-    def requestFinished(self, result, id):
+    def requestFinished(self, result, id, request):
         self.info("finished, remove %d from connection table" % id)
+        self.info("finished, sentLength: %d chunked: %d code: %d" % (request.sentLength, request.chunked, request.code))
+        self.info("finished %r" % request.headers)
         self.server.connection_manager_server.remove_connection(id)
 
     def import_file(self,name,request):
@@ -136,8 +151,13 @@ class MSRoot(resource.Resource, log.Loggable):
                                                                                 '')
                     self.info("startup, add %d to connection table" % new_id)
                     d = request.notifyFinish()
-                    d.addCallback(self.requestFinished, new_id)
-                    d.addErrback(self.requestFinished, new_id)
+                    d.addBoth(self.requestFinished, new_id, request)
+                    request.setHeader('transferMode.dlna.org', 'Streaming')
+                    if hasattr(ch.item, 'res'):
+                        if ch.item.res[0].protocolInfo is not None:
+                            _,_,_,additional_info = ch.item.res[0].protocolInfo.split(':')
+                            if additional_info != '*':
+                                request.setHeader('contentFeatures.dlna.org', additional_info)
                     return ch.location
             try:
                 p = ch.get_path()
@@ -145,7 +165,7 @@ class MSRoot(resource.Resource, log.Loggable):
                 self.debug("error accessing items path %r" % msg)
                 self.debug(traceback.format_exc())
                 return self.list_content(name, ch, request)
-            if os.path.exists(p):
+            if p != None and os.path.exists(p):
                 self.info("accessing path %r" % p)
                 new_id,_,_ = self.server.connection_manager_server.add_connection('',
                                                                             'Output',
@@ -153,9 +173,14 @@ class MSRoot(resource.Resource, log.Loggable):
                                                                             '')
                 self.info("startup, add %d to connection table" % new_id)
                 d = request.notifyFinish()
-                d.addCallback(self.requestFinished, new_id)
-                d.addErrback(self.requestFinished, new_id)
-                ch = StaticFile(p.encode('utf-8'))
+                d.addBoth(self.requestFinished, new_id, request)
+                request.setHeader('transferMode.dlna.org', 'Streaming')
+                if hasattr(ch, 'item') and hasattr(ch.item, 'res'):
+                    if ch.item.res[0].protocolInfo is not None:
+                        _,_,_,additional_info = ch.item.res[0].protocolInfo.split(':')
+                        if additional_info != '*':
+                            request.setHeader('contentFeatures.dlna.org', additional_info)
+                ch = StaticFile(p)
             else:
                 self.debug("accessing path %r failed" % p)
                 return self.list_content(name, ch, request)
@@ -163,7 +188,7 @@ class MSRoot(resource.Resource, log.Loggable):
         if ch is None:
             p = util.sibpath(__file__, name)
             if os.path.exists(p):
-                ch = StaticFile(p.encode('utf-8'))
+                ch = StaticFile(p)
         self.info('MSRoot ch', ch)
         return ch
 
@@ -186,6 +211,10 @@ class MSRoot(resource.Resource, log.Loggable):
                 elif hasattr(c,'get_path'):
                     #path = c.get_path().encode('utf-8').encode('string_escape')
                     path = c.get_path()
+                    if isinstance(path,unicode):
+                        path = path.encode('ascii','xmlcharrefreplace')
+                    else:
+                        path = path.decode('utf-8').encode('ascii','xmlcharrefreplace')
                     self.debug('has get_path', path)
                 else:
                     path = request.uri.split('/')
@@ -324,20 +353,20 @@ class RootDeviceXML(static.Data):
         self.xml = """<?xml version="1.0" encoding="utf-8"?>""" + ET.tostring( root, encoding='utf-8')
         static.Data.__init__(self, self.xml, 'text/xml')
 
-class MediaServer(log.Loggable):
+class MediaServer(log.Loggable,BasicDeviceMixin):
     logCategory = 'mediaserver'
 
     def __init__(self, coherence, backend, **kwargs):
         self.coherence = coherence
         self.device_type = 'MediaServer'
-        self.version = int(kwargs.get('version',2))
+        self.version = int(kwargs.get('version',self.coherence.config.get('version',2)))
 
         try:
             self.uuid = kwargs['uuid']
         except KeyError:
             from coherence.upnp.core.uuid import UUID
             self.uuid = UUID()
-            
+
         self.backend = None
         urlbase = self.coherence.urlbase
         if urlbase[-1] != '/':
@@ -353,6 +382,7 @@ class MediaServer(log.Loggable):
                 self.icons.append(kwargs['icon'])
 
         louie.connect( self.init_complete, 'Coherence.UPnP.Backend.init_completed', louie.Any)
+        louie.connect( self.init_failed, 'Coherence.UPnP.Backend.init_failed', louie.Any)
         reactor.callLater(0.2, self.fire, backend, **kwargs)
 
     def fire(self,backend,**kwargs):
@@ -367,7 +397,7 @@ class MediaServer(log.Loggable):
                 self.backend = backend
 
             def backend_failure(x):
-                self.warning('backend not installed, MediaServer activation aborted')
+                self.warning('backend %s not installed, MediaServer activation aborted - %s', backend, x.getErrorMessage())
                 self.debug(x)
 
             d.addCallback(backend_ready)
@@ -378,6 +408,12 @@ class MediaServer(log.Loggable):
         else:
             self.backend = backend(self, **kwargs)
 
+    def init_failed(self, backend, msg):
+        if self.backend != backend:
+            return
+        self.warning('backend not installed, MediaServer activation aborted - %s', msg.getErrorMessage())
+        self.debug(msg)
+
     def init_complete(self, backend):
         if self.backend != backend:
             return
@@ -447,78 +483,3 @@ class MediaServer(log.Loggable):
 
         self.register()
         self.info("%s MediaServer (%s) activated" % (self.backend.name, self.backend))
-
-
-    def register(self):
-        s = self.coherence.ssdp_server
-        uuid = str(self.uuid)
-        host = self.coherence.hostname
-        self.msg('%s register' % self.device_type)
-        # we need to do this after the children are there, since we send notifies
-        s.register('local',
-                    '%s::upnp:rootdevice' % uuid,
-                    'upnp:rootdevice',
-                    self.coherence.urlbase + uuid[5:] + '/' + 'description-%d.xml' % self.version,
-                    host=host)
-
-        s.register('local',
-                    uuid,
-                    uuid,
-                    self.coherence.urlbase + uuid[5:] + '/' + 'description-%d.xml' % self.version,
-                    host=host)
-
-        version = self.version
-        while version > 0:
-            if version == self.version:
-                silent = False
-            else:
-                silent = True
-            s.register('local',
-                        '%s::urn:schemas-upnp-org:device:%s:%d' % (uuid, self.device_type, version),
-                        'urn:schemas-upnp-org:device:%s:%d' % (self.device_type, version),
-                        self.coherence.urlbase + uuid[5:] + '/' + 'description-%d.xml' % version,
-                        silent=silent,
-                        host=host)
-
-            for service in self._services:
-                silencio = silent
-                if hasattr(service,'version'):
-                    if service.version < version:
-                        continue
-                    elif service.version == version:
-                        silencio = False
-                try:
-                    namespace = service.namespace
-                except:
-                    namespace = 'schemas-upnp-org'
-
-                s.register('local',
-                            '%s::urn:%s:service:%s:%d' % (uuid,namespace,service.id, version),
-                            'urn:%s:service:%s:%d' % (namespace,service.id, version),
-                            self.coherence.urlbase + uuid[5:] + '/' + 'description-%d.xml' % version,
-                            silent=silencio,
-                            host=host)
-
-            version -= 1
-
-    def unregister(self):
-        s = self.coherence.ssdp_server
-        uuid = str(self.uuid)
-        self.coherence.remove_web_resource(uuid[5:])
-
-        version = self.version
-        while version > 0:
-            s.doByebye('%s::urn:schemas-upnp-org:device:%s:%d' % (uuid, self.device_type, version))
-            for service in self._services:
-                if hasattr(service,'version') and service.version < version:
-                    continue
-                try:
-                    namespace = service.namespace
-                except AttributeError:
-                    namespace = 'schemas-upnp-org'
-                s.doByebye('%s::urn:%s:service:%s:%d' % (uuid,namespace,service.id, version))
-
-            version -= 1
-
-        s.doByebye(uuid)
-        s.doByebye('%s::upnp:rootdevice' % uuid)
\ No newline at end of file
